@@ -215,9 +215,9 @@ static BOOL add_module(struct dump_context* dc, const WCHAR* name,
     if (is_elf ||
         !GetModuleFileNameExW(dc->hProcess, (HMODULE)(DWORD_PTR)base,
                               dc->modules[dc->num_modules].name,
-                              sizeof(dc->modules[dc->num_modules].name) / sizeof(WCHAR)))
+                              ARRAY_SIZE(dc->modules[dc->num_modules].name)))
         lstrcpynW(dc->modules[dc->num_modules].name, name,
-                  sizeof(dc->modules[dc->num_modules].name) / sizeof(WCHAR));
+                  ARRAY_SIZE(dc->modules[dc->num_modules].name));
     dc->modules[dc->num_modules].base = base;
     dc->modules[dc->num_modules].size = size;
     dc->modules[dc->num_modules].timestamp = timestamp;
