@@ -263,7 +263,7 @@ static void add_tbs_to_menu(HMENU menu)
         WCHAR classes_key[] = {'S','o','f','t','w','a','r','e','\\',
                                'C','l','a','s','s','e','s','\\','C','L','S','I','D',0};
         WCHAR guid[39];
-        DWORD value_len = sizeof(guid)/sizeof(guid[0]);
+        DWORD value_len = ARRAY_SIZE(guid);
         int i;
 
         if(SHRegOpenUSKeyW(classes_key, KEY_READ, NULL, &classes_handle, TRUE) != ERROR_SUCCESS)
@@ -276,11 +276,11 @@ static void add_tbs_to_menu(HMENU menu)
         for(i = 0; SHRegEnumUSValueW(toolbar_handle, i, guid, &value_len, NULL, NULL, NULL, SHREGENUM_HKLM) == ERROR_SUCCESS; i++)
         {
             WCHAR tb_name[100];
-            DWORD tb_name_len = sizeof(tb_name)/sizeof(tb_name[0]);
+            DWORD tb_name_len = ARRAY_SIZE(tb_name);
             HUSKEY tb_class_handle;
             MENUITEMINFOW item;
             LSTATUS ret;
-            value_len = sizeof(guid)/sizeof(guid[0]);
+            value_len = ARRAY_SIZE(guid);
 
             if(lstrlenW(guid) != 38)
             {
@@ -418,7 +418,7 @@ static void add_tb_button(InternetExplorer *ie, int bmp, int cmd, int strId)
     TBBUTTON btn;
     WCHAR buf[30];
 
-    LoadStringW(ieframe_instance, strId, buf, sizeof(buf)/sizeof(buf[0]));
+    LoadStringW(ieframe_instance, strId, buf, ARRAY_SIZE(buf));
 
     btn.iBitmap = bmp;
     btn.idCommand = cmd;
@@ -445,7 +445,7 @@ static void create_rebar(InternetExplorer *ie)
     HIMAGELIST imagelist;
     SIZE toolbar_size;
 
-    LoadStringW(ieframe_instance, IDS_ADDRESS, addr, sizeof(addr)/sizeof(addr[0]));
+    LoadStringW(ieframe_instance, IDS_ADDRESS, addr, ARRAY_SIZE(addr));
 
     hwndRebar = CreateWindowExW(WS_EX_TOOLWINDOW, REBARCLASSNAMEW, NULL,
             WS_CHILD|WS_VISIBLE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN|RBS_VARHEIGHT|CCS_TOP|CCS_NODIVIDER, 0, 0, 0, 0,
