@@ -1165,7 +1165,7 @@ static void testSignSeal(void)
 
     trace("Testing with more than one buffer.\n");
 
-    crypt.cBuffers = sizeof(complex_data)/sizeof(complex_data[0]);
+    crypt.cBuffers = ARRAY_SIZE(complex_data);
     crypt.pBuffers = complex_data;
 
     complex_data[0].BufferType = SECBUFFER_DATA|SECBUFFER_READONLY_WITH_CHECKSUM;
@@ -1436,7 +1436,7 @@ static void test_cred_multiple_use(void)
             getSecError(ret));
 
     buffer_desc.ulVersion = SECBUFFER_VERSION;
-    buffer_desc.cBuffers = sizeof(buffers)/sizeof(buffers[0]);
+    buffer_desc.cBuffers = ARRAY_SIZE(buffers);
     buffer_desc.pBuffers = buffers;
 
     ret = pInitializeSecurityContextA(&cred, NULL, NULL, ISC_REQ_CONNECTION,
@@ -1487,7 +1487,7 @@ static void test_null_auth_data(void)
     buffers[0].pvBuffer = HeapAlloc(GetProcessHeap(), 0, buffers[0].cbBuffer);
 
     buffer_desc.ulVersion = SECBUFFER_VERSION;
-    buffer_desc.cBuffers = sizeof(buffers)/sizeof(buffers[0]);
+    buffer_desc.cBuffers = ARRAY_SIZE(buffers);
     buffer_desc.pBuffers = buffers;
 
     size = sizeof(user);
