@@ -1514,7 +1514,7 @@ typedef struct
     UINT container_count;
 } MetadataReaderInfo;
 
-struct metadata_container *get_metadata_container(MetadataReaderInfo *info, const GUID *guid)
+static struct metadata_container *get_metadata_container(MetadataReaderInfo *info, const GUID *guid)
 {
     unsigned i;
 
@@ -2155,7 +2155,7 @@ void ReleaseComponentInfos(void)
         IWICComponentInfo_Release(&info->IWICComponentInfo_iface);
 }
 
-HRESULT get_decoder_info(const CLSID *clsid, IWICBitmapDecoderInfo **info)
+HRESULT get_decoder_info(REFCLSID clsid, IWICBitmapDecoderInfo **info)
 {
     IWICComponentInfo *compinfo;
     HRESULT hr;
