@@ -17,9 +17,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
-#include "wine/debug.h"
-
 #include <stdarg.h>
 #include <stdlib.h>
 
@@ -34,27 +31,26 @@
 
 #include "windef.h"
 #include "winbase.h"
-#ifndef __MINGW32__
-#define USE_WS_PREFIX
-#endif
 #include "winsock2.h"
 #include "ws2ipdef.h"
+#include "ws2tcpip.h"
 #include "winhttp.h"
-#include "wincrypt.h"
 #include "winreg.h"
 #define COBJMACROS
 #include "ole2.h"
 #include "dispex.h"
 #include "activscp.h"
 
+#include "wine/debug.h"
 #include "winhttp_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(winhttp);
 
-#define DEFAULT_RESOLVE_TIMEOUT     0
-#define DEFAULT_CONNECT_TIMEOUT     20000
-#define DEFAULT_SEND_TIMEOUT        30000
-#define DEFAULT_RECEIVE_TIMEOUT     30000
+#define DEFAULT_RESOLVE_TIMEOUT             0
+#define DEFAULT_CONNECT_TIMEOUT             20000
+#define DEFAULT_SEND_TIMEOUT                30000
+#define DEFAULT_RECEIVE_TIMEOUT             30000
+#define DEFAULT_RECEIVE_RESPONSE_TIMEOUT    ~0u
 
 void set_last_error( DWORD error )
 {
@@ -106,6 +102,8 @@ static void session_destroy( object_header_t *hdr )
         domain = LIST_ENTRY( item, domain_t, entry );
         delete_domain( domain );
     }
+    session->cs.DebugInfo->Spare[0] = 0;
+    DeleteCriticalSection( &session->cs );
     heap_free( session->agent );
     heap_free( session->proxy_server );
     heap_free( session->proxy_bypass );
@@ -137,18 +135,27 @@ static BOOL session_query_option( object_header_t *hdr, DWORD option, LPVOID buf
         *(DWORD *)buffer = session->resolve_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
         *(DWORD *)buffer = session->connect_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_SEND_TIMEOUT:
         *(DWORD *)buffer = session->send_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
-        *(DWORD *)buffer = session->recv_timeout;
+        *(DWORD *)buffer = session->receive_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
+    case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        *(DWORD *)buffer = session->receive_response_timeout;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
     default:
         FIXME("unimplemented option %u\n", option);
         set_last_error( ERROR_INVALID_PARAMETER );
@@ -191,38 +198,53 @@ static BOOL session_set_option( object_header_t *hdr, DWORD option, LPVOID buffe
             set_last_error( ERROR_INSUFFICIENT_BUFFER );
             return FALSE;
         }
+        EnterCriticalSection( &session->cs );
         session->secure_protocols = *(DWORD *)buffer;
+        LeaveCriticalSection( &session->cs );
         TRACE("0x%x\n", session->secure_protocols);
         return TRUE;
     }
     case WINHTTP_OPTION_DISABLE_FEATURE:
         set_last_error( ERROR_WINHTTP_INCORRECT_HANDLE_TYPE );
         return FALSE;
+
     case WINHTTP_OPTION_RESOLVE_TIMEOUT:
         session->resolve_timeout = *(DWORD *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
         session->connect_timeout = *(DWORD *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_SEND_TIMEOUT:
         session->send_timeout = *(DWORD *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
-        session->recv_timeout = *(DWORD *)buffer;
+        session->receive_timeout = *(DWORD *)buffer;
         return TRUE;
+
+    case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        session->receive_response_timeout = *(DWORD *)buffer;
+        return TRUE;
+
     case WINHTTP_OPTION_CONFIGURE_PASSPORT_AUTH:
         FIXME("WINHTTP_OPTION_CONFIGURE_PASSPORT_AUTH: 0x%x\n", *(DWORD *)buffer);
         return TRUE;
+
     case WINHTTP_OPTION_UNLOAD_NOTIFY_EVENT:
         TRACE("WINHTTP_OPTION_UNLOAD_NOTIFY_EVENT: %p\n", *(HANDLE *)buffer);
         session->unload_event = *(HANDLE *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_MAX_CONNS_PER_SERVER:
         FIXME("WINHTTP_OPTION_MAX_CONNS_PER_SERVER: %d\n", *(DWORD *)buffer);
         return TRUE;
+
     case WINHTTP_OPTION_MAX_CONNS_PER_1_0_SERVER:
         FIXME("WINHTTP_OPTION_MAX_CONNS_PER_1_0_SERVER: %d\n", *(DWORD *)buffer);
         return TRUE;
+
     default:
         FIXME("unimplemented option %u\n", option);
         set_last_error( ERROR_WINHTTP_INVALID_OPTION );
@@ -258,8 +280,11 @@ HINTERNET WINAPI WinHttpOpen( LPCWSTR agent, DWORD access, LPCWSTR proxy, LPCWST
     session->resolve_timeout = DEFAULT_RESOLVE_TIMEOUT;
     session->connect_timeout = DEFAULT_CONNECT_TIMEOUT;
     session->send_timeout = DEFAULT_SEND_TIMEOUT;
-    session->recv_timeout = DEFAULT_RECEIVE_TIMEOUT;
+    session->receive_timeout = DEFAULT_RECEIVE_TIMEOUT;
+    session->receive_response_timeout = DEFAULT_RECEIVE_RESPONSE_TIMEOUT;
     list_init( &session->cookie_cache );
+    InitializeCriticalSection( &session->cs );
+    session->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": session.cs");
 
     if (agent && !(session->agent = strdupW( agent ))) goto end;
     if (access == WINHTTP_ACCESS_TYPE_DEFAULT_PROXY)
@@ -339,18 +364,27 @@ static BOOL connect_query_option( object_header_t *hdr, DWORD option, LPVOID buf
         *(DWORD *)buffer = connect->session->resolve_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
         *(DWORD *)buffer = connect->session->connect_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_SEND_TIMEOUT:
         *(DWORD *)buffer = connect->session->send_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
-        *(DWORD *)buffer = connect->session->recv_timeout;
+        *(DWORD *)buffer = connect->session->receive_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
+    case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        *(DWORD *)buffer = connect->session->receive_response_timeout;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
     default:
         FIXME("unimplemented option %u\n", option);
         set_last_error( ERROR_INVALID_PARAMETER );
@@ -597,6 +631,8 @@ static void request_destroy( object_header_t *hdr )
     }
     release_object( &request->connect->hdr );
 
+    CertFreeCertificateContext( request->server_cert );
+
     destroy_authinfo( request->authinfo );
     destroy_authinfo( request->proxy_authinfo );
 
@@ -648,46 +684,8 @@ static WCHAR *blob_to_str( DWORD encoding, CERT_NAME_BLOB *blob )
     return ret;
 }
 
-static BOOL convert_sockaddr( const struct sockaddr *addr, SOCKADDR_STORAGE *addr_storage )
+static BOOL copy_sockaddr( const struct sockaddr *addr, SOCKADDR_STORAGE *addr_storage )
 {
-#ifndef __MINGW32__
-    switch (addr->sa_family)
-    {
-    case AF_INET:
-    {
-        const struct sockaddr_in *addr_unix = (const struct sockaddr_in *)addr;
-        struct WS_sockaddr_in *addr_win = (struct WS_sockaddr_in *)addr_storage;
-        char *p;
-
-        addr_win->sin_family = WS_AF_INET;
-        addr_win->sin_port   = addr_unix->sin_port;
-        memcpy( &addr_win->sin_addr, &addr_unix->sin_addr, 4 );
-        p = (char *)&addr_win->sin_addr + 4;
-        memset( p, 0, sizeof(*addr_storage) - (p - (char *)addr_win) );
-        return TRUE;
-    }
-    case AF_INET6:
-    {
-        const struct sockaddr_in6 *addr_unix = (const struct sockaddr_in6 *)addr;
-        struct WS_sockaddr_in6 *addr_win = (struct WS_sockaddr_in6 *)addr_storage;
-
-        addr_win->sin6_family   = WS_AF_INET6;
-        addr_win->sin6_port     = addr_unix->sin6_port;
-        addr_win->sin6_flowinfo = addr_unix->sin6_flowinfo;
-        memcpy( &addr_win->sin6_addr, &addr_unix->sin6_addr, 16 );
-#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
-        addr_win->sin6_scope_id = addr_unix->sin6_scope_id;
-#else
-        addr_win->sin6_scope_id = 0;
-#endif
-        memset( addr_win + 1, 0, sizeof(*addr_storage) - sizeof(*addr_win) );
-        return TRUE;
-    }
-    default:
-        ERR("unhandled family %u\n", addr->sa_family);
-        return FALSE;
-    }
-#else
     switch (addr->sa_family)
     {
     case AF_INET:
@@ -710,7 +708,6 @@ static BOOL convert_sockaddr( const struct sockaddr *addr, SOCKADDR_STORAGE *add
         ERR("unhandled family %u\n", addr->sa_family);
         return FALSE;
     }
-#endif
 }
 
 static BOOL request_query_option( object_header_t *hdr, DWORD option, LPVOID buffer, LPDWORD buflen )
@@ -759,14 +756,14 @@ static BOOL request_query_option( object_header_t *hdr, DWORD option, LPVOID buf
             return FALSE;
         }
 
-        if (!request->netconn || !(cert = netconn_get_certificate( request->netconn ))) return FALSE;
+        if (!(cert = CertDuplicateCertificateContext( request->server_cert ))) return FALSE;
         *(CERT_CONTEXT **)buffer = (CERT_CONTEXT *)cert;
         *buflen = sizeof(cert);
         return TRUE;
     }
     case WINHTTP_OPTION_SECURITY_CERTIFICATE_STRUCT:
     {
-        const CERT_CONTEXT *cert;
+        const CERT_CONTEXT *cert = request->server_cert;
         const CRYPT_OID_INFO *oidInfo;
         WINHTTP_CERTIFICATE_INFO *ci = buffer;
 
@@ -778,16 +775,14 @@ static BOOL request_query_option( object_header_t *hdr, DWORD option, LPVOID buf
             set_last_error( ERROR_INSUFFICIENT_BUFFER );
             return FALSE;
         }
-        if (!request->netconn || !(cert = netconn_get_certificate( request->netconn ))) return FALSE;
+        if (!cert) return FALSE;
 
         ci->ftExpiry = cert->pCertInfo->NotAfter;
         ci->ftStart  = cert->pCertInfo->NotBefore;
         ci->lpszSubjectInfo = blob_to_str( cert->dwCertEncodingType, &cert->pCertInfo->Subject );
         ci->lpszIssuerInfo  = blob_to_str( cert->dwCertEncodingType, &cert->pCertInfo->Issuer );
         ci->lpszProtocolName      = NULL;
-        oidInfo = CryptFindOIDInfo( CRYPT_OID_INFO_OID_KEY,
-                                    cert->pCertInfo->SignatureAlgorithm.pszObjId,
-                                    0 );
+        oidInfo = CryptFindOIDInfo( CRYPT_OID_INFO_OID_KEY, cert->pCertInfo->SignatureAlgorithm.pszObjId, 0 );
         if (oidInfo)
             ci->lpszSignatureAlgName = (LPWSTR)oidInfo->pwszName;
         else
@@ -795,7 +790,6 @@ static BOOL request_query_option( object_header_t *hdr, DWORD option, LPVOID buf
         ci->lpszEncryptionAlgName = NULL;
         ci->dwKeySize = request->netconn ? netconn_get_cipher_strength( request->netconn ) : 0;
 
-        CertFreeCertificateContext( cert );
         *buflen = sizeof(*ci);
         return TRUE;
     }
@@ -831,8 +825,8 @@ static BOOL request_query_option( object_header_t *hdr, DWORD option, LPVOID buf
             return FALSE;
         }
         if (getsockname( request->netconn->socket, &local, &len )) return FALSE;
-        if (!convert_sockaddr( &local, &info->LocalAddress )) return FALSE;
-        if (!convert_sockaddr( remote, &info->RemoteAddress )) return FALSE;
+        if (!copy_sockaddr( &local, &info->LocalAddress )) return FALSE;
+        if (!copy_sockaddr( remote, &info->RemoteAddress )) return FALSE;
         info->cbSize = sizeof(*info);
         return TRUE;
     }
@@ -840,16 +834,24 @@ static BOOL request_query_option( object_header_t *hdr, DWORD option, LPVOID buf
         *(DWORD *)buffer = request->resolve_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
         *(DWORD *)buffer = request->connect_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_SEND_TIMEOUT:
         *(DWORD *)buffer = request->send_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
+
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
-        *(DWORD *)buffer = request->recv_timeout;
+        *(DWORD *)buffer = request->receive_timeout;
+        *buflen = sizeof(DWORD);
+        return TRUE;
+
+    case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        *(DWORD *)buffer = request->receive_response_timeout;
         *buflen = sizeof(DWORD);
         return TRUE;
 
@@ -972,14 +974,21 @@ static BOOL request_set_option( object_header_t *hdr, DWORD option, LPVOID buffe
     case WINHTTP_OPTION_RESOLVE_TIMEOUT:
         request->resolve_timeout = *(DWORD *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_CONNECT_TIMEOUT:
         request->connect_timeout = *(DWORD *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_SEND_TIMEOUT:
         request->send_timeout = *(DWORD *)buffer;
         return TRUE;
+
     case WINHTTP_OPTION_RECEIVE_TIMEOUT:
-        request->recv_timeout = *(DWORD *)buffer;
+        request->receive_timeout = *(DWORD *)buffer;
+        return TRUE;
+
+    case WINHTTP_OPTION_RECEIVE_RESPONSE_TIMEOUT:
+        request->receive_response_timeout = *(DWORD *)buffer;
         return TRUE;
 
     case WINHTTP_OPTION_USERNAME:
@@ -1022,6 +1031,24 @@ static BOOL request_set_option( object_header_t *hdr, DWORD option, LPVOID buffe
         }
         FIXME("WINHTTP_OPTION_CLIENT_CERT_CONTEXT\n");
         return TRUE;
+
+    case WINHTTP_OPTION_ENABLE_FEATURE:
+        if(buflen == sizeof( DWORD ) && *(DWORD *)buffer == WINHTTP_ENABLE_SSL_REVOCATION)
+        {
+            request->check_revocation = TRUE;
+            SetLastError( NO_ERROR );
+            return TRUE;
+        }
+        else
+        {
+            SetLastError( ERROR_INVALID_PARAMETER );
+            return FALSE;
+        }
+
+    case WINHTTP_OPTION_CONNECT_RETRIES:
+        FIXME("WINHTTP_OPTION_CONNECT_RETRIES\n");
+        return TRUE;
+
     default:
         FIXME("unimplemented option %u\n", option);
         set_last_error( ERROR_WINHTTP_INVALID_OPTION );
@@ -1069,6 +1096,50 @@ static BOOL store_accept_types( request_t *request, const WCHAR **accept_types )
     return TRUE;
 }
 
+static WCHAR *get_request_path( const WCHAR *object, DWORD flags )
+{
+    DWORD len, path_len = 0, query_len = 0;
+    const WCHAR *p = object, *q = NULL;
+    enum escape_flags path_flags, query_flags;
+    WCHAR *ret;
+
+    if (flags & WINHTTP_FLAG_ESCAPE_DISABLE) path_flags = ESCAPE_FLAG_SPACE_ONLY|ESCAPE_FLAG_REMOVE_CRLF;
+    else if (flags & WINHTTP_FLAG_ESCAPE_PERCENT) path_flags = ESCAPE_FLAG_PERCENT;
+    else path_flags = 0;
+
+    if (flags & WINHTTP_FLAG_ESCAPE_DISABLE_QUERY) query_flags = ESCAPE_FLAG_SPACE_ONLY|ESCAPE_FLAG_REMOVE_CRLF;
+    else query_flags = path_flags;
+
+    if (object)
+    {
+        path_len = strlenW( object );
+        if (object[0] == '/')
+        {
+            path_len--;
+            p++;
+        }
+        if ((q = strchrW( p, '?' )))
+        {
+            q++;
+            query_len = path_len - (q - p);
+            path_len -= query_len + 1;
+        }
+    }
+
+    len = escape_string( NULL, p, path_len, path_flags );
+    len += escape_string( NULL, q, query_len, query_flags );
+    if (!(ret = heap_alloc( (len + 3) * sizeof(WCHAR) ))) return NULL;
+
+    ret[0] = '/';
+    len = escape_string( ret + 1, p, path_len, path_flags ) + 1;
+    if (q)
+    {
+        ret[len] = '?';
+        escape_string( ret + 1 + len, q, query_len, query_flags );
+    }
+    return ret;
+}
+
 /***********************************************************************
  *          WinHttpOpenRequest (winhttp.@)
  */
@@ -1082,12 +1153,11 @@ HINTERNET WINAPI WinHttpOpenRequest( HINTERNET hconnect, LPCWSTR verb, LPCWSTR o
     TRACE("%p, %s, %s, %s, %s, %p, 0x%08x\n", hconnect, debugstr_w(verb), debugstr_w(object),
           debugstr_w(version), debugstr_w(referrer), types, flags);
 
-    if(types && TRACE_ON(winhttp)) {
+    if (types && TRACE_ON(winhttp))
+    {
         const WCHAR **iter;
-
         TRACE("accept types:\n");
-        for(iter = types; *iter; iter++)
-            TRACE("    %s\n", debugstr_w(*iter));
+        for (iter = types; *iter; iter++) TRACE("    %s\n", debugstr_w(*iter));
     }
 
     if (!(connect = (connect_t *)grab_object( hconnect )))
@@ -1124,25 +1194,12 @@ HINTERNET WINAPI WinHttpOpenRequest( HINTERNET hconnect, LPCWSTR verb, LPCWSTR o
     request->resolve_timeout = connect->session->resolve_timeout;
     request->connect_timeout = connect->session->connect_timeout;
     request->send_timeout = connect->session->send_timeout;
-    request->recv_timeout = connect->session->recv_timeout;
+    request->receive_timeout = connect->session->receive_timeout;
+    request->receive_response_timeout = connect->session->receive_response_timeout;
 
     if (!verb || !verb[0]) verb = getW;
     if (!(request->verb = strdupW( verb ))) goto end;
-
-    if (object)
-    {
-        WCHAR *path, *p;
-        unsigned int len;
-
-        len = strlenW( object ) + 1;
-        if (object[0] != '/') len++;
-        if (!(p = path = heap_alloc( len * sizeof(WCHAR) ))) goto end;
-
-        if (object[0] != '/') *p++ = '/';
-        strcpyW( p, object );
-        request->path = path;
-    }
-    else if (!(request->path = strdupW( slashW ))) goto end;
+    if (!(request->path = get_request_path( object, flags ))) goto end;
 
     if (!version || !version[0]) version = http1_1;
     if (!(request->version = strdupW( version ))) goto end;
@@ -1328,11 +1385,7 @@ static BOOL is_domain_suffix( const char *domain, const char *suffix )
 
 static int reverse_lookup( const struct addrinfo *ai, char *hostname, size_t len )
 {
-    int ret = -1;
-#ifdef HAVE_GETNAMEINFO
-    ret = getnameinfo( ai->ai_addr, ai->ai_addrlen, hostname, len, NULL, 0, 0 );
-#endif
-    return ret;
+    return getnameinfo( ai->ai_addr, ai->ai_addrlen, hostname, len, NULL, 0, 0 );
 }
 
 static WCHAR *build_wpad_url( const char *hostname, const struct addrinfo *ai )
@@ -1424,7 +1477,6 @@ BOOL WINAPI WinHttpDetectAutoProxyConfigUrl( DWORD flags, LPWSTR *url )
     }
     if (flags & WINHTTP_AUTO_DETECT_TYPE_DNS_A)
     {
-#ifdef HAVE_GETADDRINFO
         char *fqdn, *domain, *p;
 
         if (!(fqdn = get_computer_name( ComputerNamePhysicalDnsFullyQualified ))) return FALSE;
@@ -1466,9 +1518,6 @@ BOOL WINAPI WinHttpDetectAutoProxyConfigUrl( DWORD flags, LPWSTR *url )
         }
         heap_free( domain );
         heap_free( fqdn );
-#else
-    FIXME("getaddrinfo not found at build time\n");
-#endif
     }
     if (!ret)
     {
@@ -2062,8 +2111,6 @@ BOOL WINAPI WinHttpSetTimeouts( HINTERNET handle, int resolve, int connect, int
 {
     BOOL ret = TRUE;
     object_header_t *hdr;
-    request_t *request;
-    session_t *session;
 
     TRACE("%p, %d, %d, %d, %d\n", handle, resolve, connect, send, receive);
 
@@ -2081,43 +2128,45 @@ BOOL WINAPI WinHttpSetTimeouts( HINTERNET handle, int resolve, int connect, int
 
     switch(hdr->type)
     {
-        case WINHTTP_HANDLE_TYPE_REQUEST:
-            request = (request_t *)hdr;
-            request->connect_timeout = connect;
+    case WINHTTP_HANDLE_TYPE_REQUEST:
+    {
+        request_t *request = (request_t *)hdr;
+        request->connect_timeout = connect;
 
-            if (resolve < 0) resolve = 0;
-            request->resolve_timeout = resolve;
+        if (resolve < 0) resolve = 0;
+        request->resolve_timeout = resolve;
 
-            if (send < 0) send = 0;
-            request->send_timeout = send;
+        if (send < 0) send = 0;
+        request->send_timeout = send;
 
-            if (receive < 0) receive = 0;
-            request->recv_timeout = receive;
+        if (receive < 0) receive = 0;
+        request->receive_timeout = receive;
 
-            if (request->netconn)
-            {
-                if (netconn_set_timeout( request->netconn, TRUE, send )) ret = FALSE;
-                if (netconn_set_timeout( request->netconn, FALSE, receive )) ret = FALSE;
-            }
-            break;
+        if (request->netconn)
+        {
+            if (netconn_set_timeout( request->netconn, TRUE, send )) ret = FALSE;
+            if (netconn_set_timeout( request->netconn, FALSE, receive )) ret = FALSE;
+        }
+        break;
+    }
+    case WINHTTP_HANDLE_TYPE_SESSION:
+    {
+        session_t *session = (session_t *)hdr;
+        session->connect_timeout = connect;
 
-        case WINHTTP_HANDLE_TYPE_SESSION:
-            session = (session_t *)hdr;
-            session->connect_timeout = connect;
+        if (resolve < 0) resolve = 0;
+        session->resolve_timeout = resolve;
 
-            if (resolve < 0) resolve = 0;
-            session->resolve_timeout = resolve;
+        if (send < 0) send = 0;
+        session->send_timeout = send;
 
-            if (send < 0) send = 0;
-            session->send_timeout = send;
-
-            if (receive < 0) receive = 0;
-            session->recv_timeout = receive;
-            break;
-
-        default:
-            set_last_error( ERROR_WINHTTP_INCORRECT_HANDLE_TYPE );
-            ret = FALSE;
+        if (receive < 0) receive = 0;
+        session->receive_timeout = receive;
+        break;
+    }
+    default:
+        set_last_error( ERROR_WINHTTP_INCORRECT_HANDLE_TYPE );
+        ret = FALSE;
     }
     release_object( hdr );
     if (ret) set_last_error( ERROR_SUCCESS );
