@@ -210,6 +210,7 @@ struct options
     const char* image_base;
     const char* section_align;
     const char* lib_suffix;
+    const char* subsystem;
     strarray* prefix;
     strarray* lib_dirs;
     strarray* linker_args;
@@ -447,8 +448,12 @@ static char *get_lib_dir( struct options *opts )
 {
     static const char *stdlibpath[] = { LIBDIR, "/usr/lib", "/usr/local/lib", "/lib" };
     static const char libwine[] = "/libwine.so";
+    const char *bit_suffix, *other_bit_suffix;
     unsigned int i;
 
+    bit_suffix = opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64 ? "64" : "32";
+    other_bit_suffix = opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64 ? "32" : "64";
+
     for (i = 0; i < sizeof(stdlibpath)/sizeof(stdlibpath[0]); i++)
     {
         char *p, *buffer = xmalloc( strlen(stdlibpath[i]) + strlen("/arm-linux-gnueabi") + strlen(libwine) + 1 );
@@ -457,19 +462,15 @@ static char *get_lib_dir( struct options *opts )
         while (p > buffer && p[-1] == '/') p--;
         strcpy( p, libwine );
         if (check_platform( opts, buffer )) goto found;
-        if (p > buffer + 2 && (!memcmp( p - 2, "32", 2 ) || !memcmp( p - 2, "64", 2 ))) p -= 2;
-        if (opts->target_cpu != CPU_x86_64 && opts->target_cpu != CPU_ARM64)
+        if (p > buffer + 2 && (!memcmp( p - 2, "32", 2 ) || !memcmp( p - 2, "64", 2 )))
         {
-            strcpy( p, "32" );
-            strcat( p, libwine );
-            if (check_platform( opts, buffer )) goto found;
-        }
-        if (opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64)
-        {
-            strcpy( p, "64" );
-            strcat( p, libwine );
+            p -= 2;
+            strcpy( p, libwine );
             if (check_platform( opts, buffer )) goto found;
         }
+        strcpy( p, bit_suffix );
+        strcat( p, libwine );
+        if (check_platform( opts, buffer )) goto found;
         switch(opts->target_cpu)
         {
         case CPU_x86:     strcpy( p, "/i386-linux-gnu" ); break;
@@ -482,6 +483,37 @@ static char *get_lib_dir( struct options *opts )
         }
         strcat( p, libwine );
         if (check_platform( opts, buffer )) goto found;
+
+        strcpy( buffer, stdlibpath[i] );
+        p = buffer + strlen(buffer);
+        while (p > buffer && p[-1] == '/') p--;
+        strcpy( p, libwine );
+
+        /* try to fixup each parent dirs named lib, lib32 or lib64 with target bitness suffix */
+        while (p > buffer)
+        {
+            p--;
+            while (p > buffer && *p != '/') p--;
+            if (*p != '/') break;
+            if (memcmp( p + 1, "lib", 3 )) continue;
+            if (p[4] == '/')
+            {
+                memmove( p + 6, p + 4, strlen( p + 4 ) + 1 );
+                memcpy( p + 4, bit_suffix, 2 );
+                if (check_platform( opts, buffer )) goto found;
+                memmove( p + 4, p + 6, strlen( p + 6 ) + 1 );
+            }
+            else if (!memcmp( p + 4, other_bit_suffix, 2 ) && p[6] == '/')
+            {
+                memcpy( p + 4, bit_suffix, 2 );
+                if (check_platform( opts, buffer )) goto found;
+                memmove( p + 4, p + 6, strlen( p + 6 ) + 1 );
+                if (check_platform( opts, buffer )) goto found;
+                memmove( p + 6, p + 4, strlen( p + 4 ) + 1 );
+                memcpy( p + 4, other_bit_suffix, 2 );
+            }
+        }
+
         free( buffer );
         continue;
 
@@ -913,7 +945,19 @@ static void build(struct options* opts)
         else
         {
             strarray_add(link_args, opts->gui_app ? "-mwindows" : "-mconsole");
-            if (opts->nodefaultlibs) strarray_add(link_args, "-nodefaultlibs");
+        }
+
+        if (opts->nodefaultlibs) strarray_add(link_args, "-nodefaultlibs");
+        if (opts->nostartfiles) strarray_add(link_args, "-nostartfiles" );
+
+        if (opts->subsystem)
+        {
+            strarray_add(link_args, strmake("-Wl,--subsystem,%s", opts->subsystem));
+            if (!strcmp( opts->subsystem, "native" ))
+            {
+                const char *entry = opts->target_cpu == CPU_x86 ? "_DriverEntry@8" : "DriverEntry";
+                strarray_add(link_args, strmake( "-Wl,--entry,%s", entry ));
+            }
         }
 
         for ( j = 0 ; j < opts->linker_args->size ; j++ )
@@ -1070,6 +1114,12 @@ static void build(struct options* opts)
         if (opts->large_address_aware) strarray_add( spec_args, "--large-address-aware" );
     }
 
+    if (opts->subsystem)
+    {
+        strarray_add(spec_args, "--subsystem");
+        strarray_add(spec_args, opts->subsystem);
+    }
+
     for ( j = 0; j < lib_dirs->size; j++ )
 	strarray_add(spec_args, strmake("-L%s", lib_dirs->base[j]));
 
@@ -1211,19 +1261,11 @@ static void forward(int argc, char **argv, struct options* opts)
     strarray_free (args);
 }
 
-/*
- *      Linker Options
- *          object-file-name  -llibrary -nostartfiles  -nodefaultlibs
- *          -nostdlib -s  -static  -static-libgcc  -shared  -shared-libgcc
- *          -symbolic -Wl,option  -Xlinker option -u symbol
- *	    -framework name
- */
 static int is_linker_arg(const char* arg)
 {
     static const char* link_switches[] = 
     {
-	"-nostartfiles", "-nostdlib", "-s",
-	"-static", "-static-libgcc", "-shared", "-shared-libgcc", "-symbolic",
+	"-nostdlib", "-s", "-static", "-static-libgcc", "-shared", "-shared-libgcc", "-symbolic",
 	"-framework", "--coverage", "-fprofile-generate", "-fprofile-use"
     };
     unsigned int j;
@@ -1600,6 +1642,11 @@ int main(int argc, char **argv)
                                 opts.large_address_aware = 1;
                                 continue;
                             }
+                            if (!strcmp(Wl->base[j], "--subsystem") && j < Wl->size - 1)
+                            {
+                                opts.subsystem = strdup( Wl->base[++j] );
+                                continue;
+                            }
                             if (!strcmp(Wl->base[j], "-static")) linking = -1;
                             strarray_add(opts.linker_args, strmake("-Wl,%s",Wl->base[j]));
                         }
